/*
This is a python module that implements hci functions for blueZ

	Jos Cano Reyes

	<jocare@inf.upv.es>

This program is distributed under the GNU LGPL
See the LICENSE file or http://www.gnu.org/licenses/lgpl.txt for details
*/

#include <Python.h>

#include <stdio.h>					// general includes                                     
#include <stdlib.h>                                     
#include <unistd.h>                                     
#include <string.h>                                     
#include <signal.h>                                     
#include <fcntl.h>                                      
#include <errno.h>                                      
#include <ctype.h>

#include <sys/time.h>
#include <syslog.h>
#include <sys/select.h>
                                      
                                                        
#include <termios.h>                                    
#include <fcntl.h>                                      
#include <getopt.h>                                     
#include <sys/ioctl.h>                                  
#include <sys/socket.h>
#include <asm/types.h>
#include <netinet/in.h>

#include <bluetooth/bluetooth.h>	// bluetooth includes
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/l2cap.h>


/*********************************************************************************************************************/


// Open device

static PyObject *PyBT_open_dev(PyObject *self, PyObject *args) { 

	int	dev_id;	// device identifier
	int dd;		// device descriptor

	if ( !PyArg_ParseTuple(args, "i", &dev_id ) )	// parse incoming argument
		return NULL;

	dd = hci_open_dev( dev_id );	// call to hci function
	
	return Py_BuildValue( "i", dd );	// build a python integer and return it to python's program
}


/*********************************************************************************************************************/


// Close device

static PyObject *PyBT_close_dev(PyObject *self, PyObject *args) {

	int	dd,		// device descriptor
		ret;	// close result

	if ( !PyArg_ParseTuple(args, "i", &dd ) )	// parse incoming argument
		return NULL;

	ret = hci_close_dev( dd );	// call to hci function

	return Py_BuildValue( "i", ret );	// build a python integer and return it to python's program
}


/*********************************************************************************************************************/


// Create a connection

static PyObject *PyBT_create_connection(PyObject *self, PyObject *args) {

	int	dd,
			ret,	  // resultado
			timeout;

	char		*bdaddrstr;
	bdaddr_t	bdaddr;
	uint16_t handle;

	if ( !PyArg_ParseTuple(args, "isi", &dd, &bdaddrstr, &timeout) )
		return NULL;

	str2ba( bdaddrstr, &bdaddr );

	ret = hci_create_connection(dd, &bdaddr, 0x0008 | 0x0010, 0, 0, &handle, timeout);

	return Py_BuildValue( "i", ret );
}


/*********************************************************************************************************************/


// Disconnect

static PyObject *PyBT_disconnect(PyObject *self, PyObject *args) {

	int	dd,
			ret,	  // resultado
			timeout;

	uint16_t handle;

	if ( !PyArg_ParseTuple(args, "iii", &dd, &handle, &timeout ) )
		return NULL;

	ret = hci_disconnect(dd, handle, 0x13, timeout);

	return Py_BuildValue( "i", ret );
}


/*********************************************************************************************************************/


// Scan for bluetooth devices

static PyObject *PyBT_inquiry(PyObject *self, PyObject *args) {

	int	dev_id,
		len,
		num_rsp;

	int	flags = 0,
		i;

	inquiry_info 	*info = NULL;
	bdaddr_t 		bdaddr;

	PyObject		*bdaddrs;
	PyObject		*py_tmpbdaddrstr;

	if ( !PyArg_ParseTuple(args, "iii", &dev_id, &len, &num_rsp ) )
		return NULL;

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	num_rsp =  hci_inquiry(dev_id, len, num_rsp, NULL, &info, flags);

	if (num_rsp < 0 )
		// TODO Raise exception
		return Py_BuildValue( "s", "Query failed." );

	// return tuple of all bdaddresses
	bdaddrs = PyTuple_New( num_rsp );

	for( i = 0; i < num_rsp; i ++ ) {

		baswap(&bdaddr, &(info+i)->bdaddr);

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


		py_tmpbdaddrstr = PyString_FromString( batostr( &bdaddr ) );

		
		PyTuple_SetItem( bdaddrs, i, py_tmpbdaddrstr );
	}

	return Py_BuildValue( "(iO)", num_rsp, bdaddrs );
}



/*********************************************************************************************************************/


// Read local name

static PyObject *PyBT_read_local_name(PyObject *self, PyObject *args) {

	int			dd;
	char		name[248];

	if ( !PyArg_ParseTuple(args, "i", &dd) )	
		return NULL;


	if (hci_read_local_name(dd, sizeof(name), name, 100000) < 0)
		strcpy(name, "No tiene nombre ...");

	return Py_BuildValue( "s", name );		
}


/*********************************************************************************************************************/

// Read remote name

static PyObject *PyBT_read_remote_name(PyObject *self, PyObject *args) {

	int			dd;
	char		*bdaddrstr;

	bdaddr_t	bdaddr;

	char		name[248];

	if ( !PyArg_ParseTuple(args, "is", &dd, &bdaddrstr ) )	
		return NULL;

	str2ba( bdaddrstr, &bdaddr );


	if (hci_read_remote_name(dd, &bdaddr, sizeof(name), name, 100000) < 0)
		strcpy(name, "n/a");

	return Py_BuildValue( "s", name );		
}


/*********************************************************************************************************************/


// Return local device type

static PyObject *PyBT_local_device_type(PyObject *self, PyObject *args) {

	int			dev_id;
	struct		hci_dev_info di;
	char		*dev_type;

	if ( !PyArg_ParseTuple(args, "i", &dev_id) )		// "i" --> un integer
		return NULL;


	if (hci_devinfo(dev_id, &di) < 0)	
		return NULL;


	dev_type = hci_dtypetostr(di.type);

	return Py_BuildValue( "s", dev_type );	
}


/*********************************************************************************************************************/


// Read the device's bluetooth address

static PyObject *PyBT_local_device_ba(PyObject *self, PyObject *args) {

	int			dd;
	char		bdaddrstr[18];

	bdaddr_t	bdaddr;


	if ( !PyArg_ParseTuple(args, "i", &dd) )		// "i" --> un integer
		return NULL;


	if (hci_devba(dd, &bdaddr) < 0)
		return NULL;

	ba2str( &bdaddr, bdaddrstr );

		// convertimos la fisica bluetooth en formato string


	return Py_BuildValue( "s", bdaddrstr);	

}


/*********************************************************************************************************************/


// Switch device's role : master to slave  or  slave to master

static PyObject *PyBT_switch_role(PyObject *self, PyObject *args) {

	int		dd, role;
	char		*bdaddrstr;
	
	bdaddr_t	bdaddr;

	if ( !PyArg_ParseTuple(args, "isi", &dd, &bdaddrstr, &role) )
		return NULL;

	str2ba( bdaddrstr, &bdaddr );

	if (hci_switch_role(dd, bdaddr, role, 10000) < 0)		// role: 0 --> master , 1 --> slave
		return Py_BuildValue( "s", "FAIL" );

	return Py_BuildValue( "s", "OK" );		
}


/*********************************************************************************************************************/

// Define methods to export

static PyMethodDef BluetoothMethods[] = {

	{"open_dev",  PyBT_open_dev, METH_VARARGS, "Open device"},
	{"close_dev",  PyBT_close_dev, METH_VARARGS, "Close device"},
	{"create_connection",  PyBT_create_connection, METH_VARARGS, "Create a connection"},
	{"disconnect",  PyBT_disconnect, METH_VARARGS, "Disconnect"},
	{"inquiry",  PyBT_inquiry, METH_VARARGS, "Scan for bluetooth devices"},
	{"read_local_name",  PyBT_read_local_name, METH_VARARGS, "Get local device's name"},
	{"read_remote_name",  PyBT_read_remote_name, METH_VARARGS, "Get remote device's name"},
	{"local_device_type",  PyBT_local_device_type, METH_VARARGS, "Get local device's type "},
	{"local_device_ba",  PyBT_local_device_ba, METH_VARARGS, "Get local device's address"},
	{"switch_role",  PyBT_switch_role, METH_VARARGS, "Change device's role"},
	{NULL, NULL, 0, NULL}        // Sentinel

};


/*********************************************************************************************************************/


void initbluezhci( void ) {
	
	// Create a new module object based on a name and table of functions, returning the new module object
	
	Py_InitModule("bluezhci", BluetoothMethods);

}




